@@ -66,6 +66,15 @@ namespace Beagle {
 
 		//////////////////////////
 
+		static private XmlSerializer our_serializer;
+
+		static Indexable ()
+		{
+			our_serializer = new XmlSerializer (typeof (Indexable));
+		}
+
+		//////////////////////////
+
 		public Indexable (Uri _uri) {
 			uri = _uri;
 
@@ -79,8 +88,7 @@ namespace Beagle {
 		public static Indexable NewFromXml (string xml)
 		{
 			StringReader reader = new StringReader (xml);
-			XmlSerializer s = new XmlSerializer (typeof (Indexable));
-			return (Indexable)s.Deserialize (reader);
+			return (Indexable) our_serializer.Deserialize (reader);
 		}
 
 		//////////////////////////
@@ -154,9 +162,32 @@ namespace Beagle {
 		}
 
 		//////////////////////////
+		
+		private TextReader ReaderFromUri (Uri uri)
+		{
+			TextReader reader = null;
+
+			if (uri != null && uri.IsFile) {
+				Stream stream = new FileStream (uri.LocalPath,
+								FileMode.Open,
+								FileAccess.Read,
+								FileShare.Read);
+
+				reader = new StreamReader (stream);
+
+				// Paranoia: never delete the thing we are actually indexing.
+				if (DeleteContent && uri != Uri)
+					File.Delete (uri.LocalPath);
+			}
+
+			return reader;
+		}
 
 		public virtual TextReader GetTextReader ()
 		{
+			if (textReader == null)
+				textReader = ReaderFromUri (ContentUri);
+
 			return textReader;
 		}
 		
@@ -167,6 +198,8 @@ namespace Beagle {
 
 		public virtual TextReader GetHotTextReader ()
 		{
+			if (hotTextReader == null)
+				hotTextReader = ReaderFromUri (HotContentUri);
 			return hotTextReader;
 		}
 
@@ -208,11 +241,11 @@ namespace Beagle {
 		}
 		
 		//////////////////////////
+
 		public override string ToString () 
 		{
-			XmlSerializer s = new XmlSerializer (typeof (Indexable));
 			StringWriter writer = new StringWriter ();
-			s.Serialize (writer, this);
+			our_serializer.Serialize (writer, this);
 			writer.Close ();
 			return writer.ToString ();
 		}
@@ -243,13 +276,15 @@ namespace Beagle {
 		}
 
 		public void StoreStream () {
-			if (textReader != null)
+			if (textReader != null) {
 				ContentUri = TextReaderToTempFileUri (textReader);
+				DeleteContent = true;
+			}
 
-			if (hotTextReader != null)
+			if (hotTextReader != null) {
 				HotContentUri = TextReaderToTempFileUri (hotTextReader);
-
-			DeleteContent = true;
+				DeleteContent = true;
+			}
 		}
 
 		//////////////////////////
