@@ -1,6 +1,6 @@
 /* -*- Mode: csharp; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 //
-// FilterExcel.cs
+// FilterPPT.cs
 //
 // Copyright (C) 2004 Novell, Inc.
 //
@@ -32,6 +32,8 @@ using System.IO;
 using System.Text;
 using Gsf;
 
+using Beagle.Util;
+
 internal class RecordType
 {
 
@@ -369,17 +371,33 @@ internal class RecordType
 
 namespace Beagle.Filters {
     
-	public class FilterPPT : FilterOle {
+	public class FilterPPT : Beagle.Daemon.Filter {
 
-		bool read_text;
-		int slide_count;
+		private enum TextType {
+			Invalid = -1,
+			Title,
+			Body,
+			Notes,
+			NotUsed,
+			Other,
+			CenterBody,
+			CenterTitle,
+			HalfBody,
+			QuarterBody
+		};
 
+		Infile file;
+		TextType textType;
+		string FileName;
 		public FilterPPT () 
 		{
 			AddSupportedMimeType ("application/vnd.ms-powerpoint");
+			textType = TextType.Invalid;
+			file = null;
+			FileName = null;
 		}
 
-		private int ParseElement (Gsf.Input stream)
+		private int ParseElement (Input stream)
 		{
 			byte [] data = stream.Read(8);
 			if (data == null)
@@ -387,59 +405,215 @@ namespace Beagle.Filters {
 			RecordType.TypeCode opcode = (RecordType.TypeCode) GetInt16(data, 2);
 			int length = GetInt32(data, 4);
 			RecordType type = RecordType.Find (opcode);
+
+			// Process the container tree
 			if (type.is_container) {
 				int length_remaining = length;
-				if (opcode == RecordType.TypeCode.SlideListWithText) {
-					slide_count = 0;
-				}
-				while (length_remaining > 0) {
-					int elem_length = ParseElement(stream);
-					if (elem_length == 0)
-						return 0;
-					length_remaining -= elem_length;
+
+				if (opcode == RecordType.TypeCode.MainMaster) {
+					// Ignore MainMaster container as it contains
+					// just a master-slide view and no user data.
+					stream.Seek (length_remaining, SeekOrigin.Current);
+				} else {
+					while (length_remaining > 0) {
+						int elem_length = ParseElement(stream);
+						if (elem_length == 0)
+							return 0;
+						length_remaining -= elem_length;
+					}
 				}
 			} else {
 				if (length != 0) {
 					System.Text.Encoding encoding = null;
 
 					if (opcode == RecordType.TypeCode.TextBytesAtom) {
-						encoding = System.Text.Encoding.GetEncoding (28591);
+						//encoding = System.Text.Encoding.GetEncoding (28591);
+						encoding = System.Text.Encoding.UTF8;
 					} else if (opcode == RecordType.TypeCode.TextCharsAtom) {
 						encoding = System.Text.Encoding.Unicode;
-					} else if (opcode == RecordType.TypeCode.SlidePersistAtom) {
-						slide_count ++;
 					}
-
-					if (encoding != null && read_text) {
+					
+					if (encoding != null && textType != TextType.NotUsed) {
+						StringBuilder strData = new StringBuilder () ;
 						data = stream.Read(length);
 						if (data == null)
 							return 0;
-						AppendText (encoding.GetString (data));
-						AppendWhiteSpace ();
+						// Replace all ^M with "whitespace",
+						// because of which the contents were not properly 
+						// been appended to the text pool.
+						strData.Append (encoding.GetString (data).Replace ('\r', ' '));
+
+						// Replace all ^K with "whitespace",
+						// because of which the contents were not properly 
+						// been appended to the text pool.
+						strData.Replace ((char)0x0B, (char)0x20);
+						
+						if (textType == TextType.Title ||
+						    textType == TextType.CenterBody ||
+						    textType == TextType.CenterTitle)
+							HotUp ();
+						AppendText (strData.ToString());
+						if (IsHot)
+							HotDown ();
+						else
+							AppendWhiteSpace ();
+					}  else if (opcode == RecordType.TypeCode.TextHeaderAtom) {
+						data = stream.Read (4);
+						textType = (TextType) GetInt32 (data, 0);
 					} else {
 						stream.Seek(length, SeekOrigin.Current);
 					}
 				}
 			}
+
+			// length = RecordHeader.recLen
+			// 8 = sizeof (RecordHeader)
+			// Every Atom/container is preceded by a RecordHeader
 			return length + 8;
 		}
+		
 
+		private void ExtractMetaData (Input sumStream, Input docSumStream)
+		{
+			DocMetaData sumMeta = null;
+			if (sumStream != null)
+				sumMeta = Msole.MetadataReadReal (sumStream);
+			else
+				Logger.Log.Error ("SummaryInformationStream not found in {0}", FileName);
+			
+			DocMetaData docSumMeta = null;
+			if (docSumStream != null)
+				docSumMeta = Msole.MetadataReadReal (docSumStream);
+			else
+				Logger.Log.Error ("DocumentSummaryInformationStream not found in {0}", FileName);
+
+			DocProp prop = null;
+			string str = null;
+			if (sumMeta != null) {
+				prop = sumMeta.GetProp ("dc:title");
+				if (prop != null)
+					str = Gsf.Global.GetPropValStr (prop);
+				if (str != null && str.Length > 0)
+					AddProperty (Beagle.Property.New ("dc:title", str));
+
+				str = null;
+				prop = sumMeta.GetProp ("dc:subject");			
+				if (prop != null)
+					str = Gsf.Global.GetPropValStr (prop);			
+				if (str != null && str.Length > 0)
+					AddProperty (Beagle.Property.New ("dc:subject", str));
+
+				str = null;
+				prop = sumMeta.GetProp ("dc:description");		
+				if (prop != null)
+					str = Gsf.Global.GetPropValStr (prop);			
+				if (str != null && str.Length > 0)
+					AddProperty (Beagle.Property.New ("dc:description", str));
+
+				str = null;
+				prop = sumMeta.GetProp ("gsf:keywords");
+				if (prop != null)
+					str = Gsf.Global.GetPropValStr (prop);
+				if (str != null && str.Length > 0)
+					AddProperty (Beagle.Property.New ("fixme:keywords", str));
+
+				str = null;
+				prop = sumMeta.GetProp ("gsf:creator");
+				if (prop != null)
+					str = Gsf.Global.GetPropValStr (prop);
+				if (str != null && str.Length > 0)
+					AddProperty (Beagle.Property.New ("fixme:author", str));
+			}
+			
+			if (docSumMeta != null) {
+				str = null;
+				prop = docSumMeta.GetProp ("gsf:company");
+				if (prop != null)
+					str = Gsf.Global.GetPropValStr (prop);
+				if (str != null && str.Length > 0)
+					AddProperty (Beagle.Property.New ("fixme:company", str));
+
+				str = null;
+				prop = docSumMeta.GetProp ("gsf:slide-count");
+				if (prop != null)
+					str = Gsf.Global.GetPropValStr (prop);
+				if (str != null && str.Length > 0)
+					AddProperty (Beagle.Property.New ("fixme:slide-count", str));
+			}
+		}
 		override protected void DoPullProperties ()
 		{
-			Input stream = file.ChildByName ("PowerPoint Document");
+			Input sumStream = null;
+			Input docSumStream = null;
+			string str = null;
+			int childCount = 0;
+			int found = 0;
+			
+			if (file == null)
+				return;
 
-			read_text = false;
-			ParseElement(stream);
-			AddProperty (Beagle.Property.New ("fixme:SlideCount", slide_count));
+			childCount = file.NumChildren();
+			for (int i = 0; i < childCount && found != 2; i++) {
+				str = file.NameByIndex (i);
+				if (string.Compare (str, "SummaryInformation") == 0) {
+					sumStream = file.ChildByIndex (i);
+					found = 1;
+				}
+				else if (string.Compare (str, "DocumentSummaryInformation") == 0) {
+					docSumStream = file.ChildByIndex (i);
+					found = 2;
+				}
+			}
+			ExtractMetaData (sumStream, docSumStream);
 		}
 
 		override protected void DoPull ()
 		{
-			read_text = true;
-			Input stream = file.ChildByName ("PowerPoint Document");
+			if (file == null)
+				return;
 
-			ParseElement(stream);
+			Input stream = null;
+			stream = file.ChildByName ("PowerPoint Document");
+
+			if (stream != null) {
+
+				// The parsing was getting terminated when "EndDocument"
+				// container was parsed.  We need to continue our 
+				// parsing till the end of the file, since, some of the
+				// slides do persist after the actual "Document" 
+				// container.
+				// PPTs exported from OO.o actually writes almost all the slides
+				// after "Document" container.
+				// And certain PPTs do have some slides in after
+				// "Document" container.
+				while (!stream.Eof)
+					ParseElement (stream);
+			} else 
+				Logger.Log.Error ("Ole stream not found in {0}.  Content extraction skipped.", FileName);
 			Finished();
 		}
+
+		override protected void DoOpen (FileInfo info)
+		{
+			FileName = info.FullName;
+
+			Gsf.Global.Init ();
+			Input input = Input.MmapNew (info.FullName);
+			if (input != null) {
+				input = input.Uncompress();
+				file = new InfileMSOle (input);
+			} else 
+				Logger.Log.Error ("Unable to open {0}", info.FullName);
+		}
+
+		// FIXME: These are utility functions and can be useful 
+		// outside this filter as well.
+		public static int GetInt32 (byte [] data, int offset) {
+			return data[offset] + (data[offset + 1] << 8) + (data[offset + 2] << 16) + (data[offset + 3] << 24);
+		}
+		public static int GetInt16 (byte [] data, int offset) {
+			return data[offset] + (data[offset + 1] << 8);
+		}
+
 	}
 }
