@@ -123,7 +123,6 @@ namespace Beagle.Filters {
 		int skipCount;
 		int hotStyleCount;
 		bool bPartHotStyle;
-		long offset;
 		FileStream FsRTF;
 		StreamReader SReaderRTF;
 	        string partText;
@@ -140,7 +139,6 @@ namespace Beagle.Filters {
 			skipCount = 0;
 			hotStyleCount = 0;
 			bPartHotStyle = false;
-			offset = 0;
 			FsRTF = null;
 			SReaderRTF = null;
 			partText = "";
@@ -311,10 +309,9 @@ namespace Beagle.Filters {
 			ErrorCodes ec;
 			int OriginalGrpCount = 0x7FFFFFFF;
 
-			// If we are not extracting meta-data, set the 
-			// file pointer to the saved position
-			if (!bMeta)
-				SReaderRTF.BaseStream.Seek (offset, SeekOrigin.Begin);
+			// "/info" can be anywhere in the document,
+			// so, rewind the file pointer to start from beginning.
+			SReaderRTF.BaseStream.Seek (0, SeekOrigin.Begin);
 		       
 			while ((aByte = SReaderRTF.Read ()) != -1) {
 				ch = (char) aByte;
@@ -346,11 +343,8 @@ namespace Beagle.Filters {
 						// groupCount will atleast be 1 for 
 						// the outermost "{" block
 						if (pos == Position.InMetaData && groupCount == 1) {
-							if (bMeta) {
-								offset = SReaderRTF.BaseStream.Position;
+							if (bMeta)
 								return ErrorCodes.ERROR_RTF_OK;
-							}
-
 						} else {
 							if (MetaDataStack.Count > 0) {
 								strTemp = (string) MetaDataStack.Pop ();
@@ -374,6 +368,7 @@ namespace Beagle.Filters {
 						if (str.Length > 0)
 							str.Append (' ');
 						AddTextForIndexing (str);
+						AppendStructuralBreak ();
 					}
 					if (hotStyleCount > 0
 					    && groupCount <= hotStyleCount) {
@@ -432,6 +427,9 @@ namespace Beagle.Filters {
 				int sindex = 0;
 
 				if (index > -1) {
+					// During the previous-parsing, a word got terminatted partially,
+					// find the remaining part of the word, concatenate it and add it to 
+					// the respective pools and reset the HOT status, if required.
 					if (partText.Length > 0) {
 						sindex = paramStr.IndexOf (' ');
 						strTemp = partText + paramStr.Substring (0, sindex);
