@@ -36,6 +36,7 @@ namespace Beagle.Filters {
 		{
 			AddSupportedMimeType ("text/plain");
 			SnippetMode = true;
+			OriginalIsText = true;
 		}
 
 		override protected void DoPull ()
@@ -43,9 +44,9 @@ namespace Beagle.Filters {
 			string str = TextReader.ReadLine ();
 			if (str == null) {
 				Finished ();
-			} else {
+			} else if (str.Length > 0) {
 				AppendText (str);
-				AppendWhiteSpace ();
+				AppendStructuralBreak ();
 			}
 		}
 	}
