@@ -97,10 +97,12 @@ namespace Beagle.Tile {
 			string command = null;
 			string commandArgs = null;
 
-			if (uri.StartsWith ("http://") || uri.StartsWith ("file://")) {
+			if (uri.StartsWith (Uri.UriSchemeHttp)
+			    || uri.StartsWith (Uri.UriSchemeHttps)
+			    || uri.StartsWith (Uri.UriSchemeFile)) {
 				command = "gnome-open";
 				commandArgs = "'" + uri + "'";
-			} else if (uri.StartsWith ("mailto:")) {
+			} else if (uri.StartsWith (Uri.UriSchemeMailto)) {
 				command = "evolution";
 				commandArgs = uri;
 			}
@@ -265,15 +267,22 @@ namespace Beagle.Tile {
 			TileCanvasRenderContext ctx;
 			ctx = new TileCanvasRenderContext (this, 
 							   tile);
-			ctx.Write ("<html><style type=\"text/css\" media=\"screen\">");
 
+			// IMPORTANT!  The <meta> tag has to be in the first
+			// chunk written to the stream or else the encoding
+			// will be wrong!
+			ctx.Write ("<html>\n<head>\n<meta http-equiv=\"content-type\" content=\"text/html; charset=UTF-8\">\n");
+
+       			ctx.Write ("<style type=\"text/css\" media=\"screen\">\n");
 			RenderStyles (ctx);
-			
-			ctx.Write ("</style>");
+			ctx.Write ("</style>\n"); 
+
+			ctx.Write ("</head>\n");
 
 			if (tile != null) {
 				tile.Render (ctx);
 			}
+
 			ctx.Write ("</html>");
 		}
 		
@@ -315,7 +324,7 @@ namespace Beagle.Tile {
 			string mime_type = "text/html";
 			if (Environment.GetEnvironmentVariable ("BEST_DEBUG_HTML") != null)
 			    mime_type = "text/plain";
-			OpenStream ("http://localhost/", mime_type);
+			OpenStream ("file:///", mime_type);
 			PaintTile (root);
 			CloseStream ();
 
