@@ -27,31 +27,35 @@
 using System;
 using System.IO;
 using System.Collections;
-using BU = Beagle.Util;
+using System.Diagnostics;
+using Beagle.Util;
 
 namespace Beagle.Tile {
 
 	[HitFlavor (Name="Conversations", Rank=900, Emblem="emblem-im-log.png", Color="#e5f5ef",
 		    Type="IMLog")]
 	public class TileImLog : TileFromHitTemplate {
-		private static BU.GaimBuddyListReader list = null;
+		private static GaimBuddyListReader list = null;
 
 #if ENABLE_EVO_SHARP
 		private static Hashtable buddy_emails = new Hashtable ();
 #endif
 
-		private BU.ImBuddy buddy = null;
+		private ImBuddy buddy = null;
 		private string email = null;
 
 		public TileImLog (Hit _hit) : base (_hit,
 						    "template-im-log.html")
 		{
 			if (list == null) {
-				list = new BU.GaimBuddyListReader ();
+				list = new GaimBuddyListReader ();
 			}
 
 			buddy = list.Search (Hit ["fixme:speakingto"]);
-			email = GetEmailForIm (Hit ["fixme:speakingto"]);
+			if (buddy != null) {
+				Console.WriteLine ("Found buddy info for {0}, icon at {1}", buddy.Alias, buddy.BuddyIconLocation);
+				email = GetEmailForName (buddy.Alias);
+			}
 		}
 
 		protected override void PopulateTemplate ()
@@ -59,18 +63,15 @@ namespace Beagle.Tile {
 			base.PopulateTemplate ();
 
 			Template["nice_duration"] = "(" +
-				BU.StringFu.DurationToPrettyString (
-					   BU.StringFu.StringToDateTime (Hit ["fixme:endtime"]),
-					   BU.StringFu.StringToDateTime (Hit ["fixme:starttime"])) + ")";
+				StringFu.DurationToPrettyString (
+					StringFu.StringToDateTime (Hit ["fixme:endtime"]),
+					StringFu.StringToDateTime (Hit ["fixme:starttime"])) + ")";
 			if (Template ["nice_duration"] == "()")
 				Template ["nice_duration"] = "";
 
 			if (email != null)
 				Template ["SendMailAction"] = "Send Mail";
 			
-#if false
-			Template["snippet"] = getSnippet ();
-#endif
 			if (buddy != null && buddy.Alias != "")
 				Template["speakingalias"] = buddy.Alias;
 			else 
@@ -82,10 +83,9 @@ namespace Beagle.Tile {
 				fullpath = Path.Combine (fullpath, "icons");
 				fullpath = Path.Combine (fullpath, buddy.BuddyIconLocation);
 
-				Console.WriteLine ("Icon for {0}: {1}", buddy.Alias, fullpath);
-
 				if (File.Exists (fullpath)) {
-					Template["Icon"] = "file://" + fullpath;				} else {
+					Template["Icon"] = StringFu.PathToQuotedFileUri (fullpath);
+				} else {
 					Template["Icon"] = Images.GetHtmlSourceForStock ("gnome-gaim", 48);
 				}
 			} else {
@@ -97,18 +97,24 @@ namespace Beagle.Tile {
 		static bool ebook_failed = false;
 #endif
 
-		private string GetEmailForIm (string im)
+		private string GetEmailForName (string name)
 		{
 #if ENABLE_EVO_SHARP
+			if (name == null || name == "")
+				return null;
+
 			Evolution.Book addressbook = null;
 
+
 			// If we've previously failed to open the
 			// addressbook, don't keep trying.
 			if (ebook_failed)
 				return null;
 
-			if (buddy_emails.Contains (im)) {
-				string str = (string)buddy_emails[im];
+			// We keep a little cache so we don't have to query
+			// the addressbook too often.
+			if (buddy_emails.Contains (name)) {
+				string str = (string)buddy_emails[name];
 				return str != "" ? str : null;
 			}
 
@@ -124,24 +130,19 @@ namespace Beagle.Tile {
 
 			// Do a search.
 			string qstr =
-				String.Format ("(or " +
-					           "(is \"im_aim\" \"{0}\") " + 
-					           "(is \"im_yahoo\" \"{0}\") " +
-					           "(is \"im_msn\" \"{0}\") " + 
-					           "(is \"im_icq\" \"{0}\") " +
-					           "(is \"im_jabber\" \"{0}\") " + 
-					           "(is \"im_groupwise\" \"{0}\") " +
-					       ")",
-					       im);
+				String.Format ("(is \"full_name\" \"{0}\")", name);
 
 			Evolution.BookQuery query = Evolution.BookQuery.FromString (qstr);
 			Evolution.Contact [] matches = addressbook.GetContacts (query);
 			foreach (Evolution.Contact c in matches) {
-				Console.WriteLine ("Got match: " + c.FullName);
-				buddy_emails[im] = c.Email1;
-				return c.Email1;
+				Console.WriteLine ("FIXME: querying the evolution addressbook instead of using Lucene, this is slow and dumb");
+				Console.WriteLine ("Got match: {0} {1}", c.FullName, c.Email1);
+				if (c.Email1 != null) {
+					buddy_emails[name] = c.Email1;
+					return c.Email1;
+				}
 			}
-			buddy_emails[im] = "";
+			buddy_emails[name] = "";
 #endif
 
 			return null;
@@ -189,31 +190,16 @@ namespace Beagle.Tile {
 				return null;
 		}
 
-		private string getSnippet ()
+		[TileAction]
+		public override void Open ()
 		{
-			ICollection logs = BU.GaimLog.ScanLog (new FileInfo (Hit ["fixme:file"]));
-
-			string snip = "";
-			
-			foreach (BU.ImLog log in logs) {
-					foreach (BU.ImLog.Utterance utt in log.Utterances) {
-						// FIXME: Query.Text is broken for me
-#if false
-						string s = HighlightOrNull (utt.Text, Query.Text);
-#else
-						string s = utt.Text;
-#endif
-						if (s != null) {
-							if (snip != "")
-								snip += " ... ";							    
-							snip += s;
-						}
-						
-					}
-			}
-
-			return snip.Substring (0, System.Math.Min (256, snip.Length));
+			//FIXME: At least for now
+			Process p = new Process ();
+			p.StartInfo.UseShellExecute = false;
+			p.StartInfo.FileName = "beagle-imlogviewer";
+			p.StartInfo.Arguments = Hit ["fixme:file"];
 
+			p.Start ();
 		}
 
 		[TileAction]
