@@ -243,14 +243,14 @@ namespace Beagle.Daemon {
 			while (di != null) {
 				// Evo uses ".sbd" as the extension on a folder
 				if (di.Extension == ".sbd")
-					folder_name = Path.Combine (folder_name, Path.GetFileNameWithoutExtension (di.Name));
+					folder_name = Path.Combine (Path.GetFileNameWithoutExtension (di.Name), folder_name);
 				else
 					break;
 
 				di = di.Parent;
 			}
 
-			return Path.Combine (folder_name, Path.GetFileNameWithoutExtension (file_info.Name));
+			return Path.Combine (folder_name, file_info.Name);
 		}
 
 		protected override bool Setup ()
