@@ -37,51 +37,114 @@ namespace Beagle.Daemon {
 
 	public class Filter {
 
+		public enum ContentType {
+			Text,
+			WhiteSpace,
+			StructuralBreak
+		}
+
+		public delegate void GotPropertyHandler (Property prop);
+		public delegate void GotContentHandler  (ContentType type, string content);
+
+		public event GotPropertyHandler GotPropetyEvent;
+		public event GotContentHandler  GotContentEvent;
+
+		//////////////////////////
+
 		// Derived classes always must have a constructor that
 		// takes no arguments.
 		public Filter () { }
 
 		//////////////////////////
 
-		private   ArrayList supportedFlavors = new ArrayList ();
-		protected Flavor flavor;
+		private string identifier;
 
-		protected void AddSupportedFlavor (Flavor flavor)
-		{
-			supportedFlavors.Add (flavor);
-		}
-
-		protected void AddSupportedMimeType (String mimeType)
-		{
-			Flavor flavor = new Flavor (mimeType, Flavor.Wildcard);
-			AddSupportedFlavor (flavor);
-		}
-
-		protected void AddSupportedExtension (String extension)
-		{
-			Flavor flavor = new Flavor (Flavor.Wildcard, extension);
-			AddSupportedFlavor (flavor);
-		}
-
-		public IEnumerable SupportedFlavors {
-			get { return supportedFlavors; }
-		}
-
-		public Flavor Flavor {
-			get { return flavor; }
+		public string Identifier {
+			get { return identifier; }
+			set { identifier = value; }
 		}
 
 		//////////////////////////
 
-		private bool crawlMode = false;
+		private   ArrayList supported_mime_types = new ArrayList ();
+		private   ArrayList supported_extensions = new ArrayList ();
+		
+		protected void AddSupportedMimeType (string mime_type)
+		{
+			supported_mime_types.Add (mime_type);
+		}
+
+		protected void AddSupportedExtension (string extension)
+		{
+			supported_extensions.Add (extension);
+		}
+
+		public IEnumerable SupportedMimeTypes {
+			get { return supported_mime_types; }
+		}
+
+		public IEnumerable SupportedExtensions {
+			get { return supported_mime_types; }
+		}
+
+		//////////////////////////
+
+		// Filters are versioned.  This allows us to automatically re-index
+		// files when a newer filter is available.
+
+		public string Name {
+			get { return this.GetType ().Name; }
+		}
+
+		private int version = -1;
+
+		public int Version {
+			get { return version < 0 ? 0 : version; }
+		}
+
+		protected void SetVersion (int v)
+		{
+			if (v < 0) {
+				string msg;
+				msg = String.Format ("Attempt to set invalid version {0} on Filter {1}", v, Name);
+				throw new Exception (msg);
+			}
+
+			if (version != -1) {
+				string msg;
+				msg = String.Format ("Attempt to re-set version from {0} to {1} on Filter {2}", version, v, Name);
+				throw new Exception (msg);
+			}
+
+			version = v;
+		}
+
+		//////////////////////////
+
+		private string this_mime_type = null;
+		private string this_extension = null;
+
+		public string MimeType {
+			get { return this_mime_type; }
+			set { this_mime_type = value; }
+		}
+
+		public string Extension {
+			get { return this_extension; }
+			set { this_extension = value; }
+		}
+
+		//////////////////////////
+		
+		private bool crawl_mode = false;
 
 		public void EnableCrawlMode ()
 		{
-			crawlMode = true;
+			crawl_mode = true;
 		}
 		
 		protected bool CrawlMode {
-			get { return crawlMode; }
+			get { return crawl_mode; }
 		}
 
 		//////////////////////////
@@ -96,11 +159,8 @@ namespace Beagle.Daemon {
 		
 		public void HotDown ()
 		{
-			if (hotCount > 0) {
+			if (hotCount > 0)
 				--hotCount;
-				if (hotCount == 0)
-					AppendWhiteSpace ();
-			}
 		}
 
 		public bool IsHot {
@@ -114,7 +174,8 @@ namespace Beagle.Daemon {
 
 		public void FreezeDown ()
 		{
-			--freezeCount;
+			if (freezeCount > 0)
+				--freezeCount;
 		}
 
 		public bool IsFrozen {
@@ -124,12 +185,18 @@ namespace Beagle.Daemon {
 		//////////////////////////
 		
 		private bool snippetMode = false;
+		private bool originalIsText = false;
 		private TextWriter snippetWriter = null;
 
 		public bool SnippetMode {
 			get { return snippetMode; }
 			set { snippetMode = value; }
 		}
+
+		public bool OriginalIsText {
+			get { return originalIsText; }
+			set { originalIsText = value; }
+		}
 		
 		public void AttachSnippetWriter (TextWriter writer)
 		{
@@ -142,6 +209,13 @@ namespace Beagle.Daemon {
 		private ArrayList textPool;
 		private ArrayList hotPool;
 		private ArrayList propertyPool;
+		
+		private int max_text_pool_len = 0;
+		private int max_text_pool_size = 0;
+		private int max_hot_pool_len = 0;
+		
+
+		private bool last_was_structural_break = true;
 
 		public void AppendText (string str)
 		{
@@ -149,13 +223,34 @@ namespace Beagle.Daemon {
 			if (! IsFrozen && str != null && str != "") {
 
 				// FIXME: We should be smarter about newlines
-				str = str.Replace ("\n", " ");
+				if (str.IndexOf ('\n') != -1)
+					str = str.Replace ("\n", " ");
 
 				textPool.Add (str);
+
+				// For Hot pool, Add the "whitespace"
+				// while adding text to it. This way
+				// we can avoid extra spaces.
 				if (IsHot)
-					hotPool.Add (str);
+					hotPool.Add (str.Trim()+" ");
+
+				int pool_size = 0;
+				foreach (string x in textPool)
+					pool_size += x.Length;
+
+				if (pool_size > max_text_pool_size)
+					max_text_pool_size = pool_size;
+
+				if (textPool.Count > max_text_pool_len)
+					max_text_pool_len = textPool.Count;
+
+				if (hotPool.Count > max_hot_pool_len)
+					max_hot_pool_len = hotPool.Count;
+
 				if (snippetWriter != null)
 					snippetWriter.Write (str);
+
+				last_was_structural_break = false;
 			}
 		}
 
@@ -174,15 +269,16 @@ namespace Beagle.Daemon {
 
 		public void AppendWhiteSpace ()
 		{
+			if (last_was_structural_break)
+				return;
+
 			//Logger.Log.Debug ("AppendWhiteSpace ()");
 			if (NeedsWhiteSpace (textPool)) {
 				textPool.Add (" ");
 				if (snippetWriter != null)
 					snippetWriter.Write (" ");
+				last_was_structural_break = false;
 			}
-			if (NeedsWhiteSpace (hotPool))
-				hotPool.Add (" ");
-			
 		}
 
 		public void AddProperty (Property prop)
@@ -191,10 +287,16 @@ namespace Beagle.Daemon {
 				propertyPool.Add (prop);
 		}
 
-		public void AddStructuralBreak ()
+		public void AppendStructuralBreak ()
 		{
-			if (snippetWriter != null)
+			if (snippetWriter != null && ! last_was_structural_break) {
 				snippetWriter.WriteLine ();
+				last_was_structural_break = true;
+			}
+			// When adding a "newline" to the textCache, we need to 
+			// append a "Whitespace" to the text pool.
+			if (NeedsWhiteSpace (textPool))
+				textPool.Add (" ");
 		}
 
 		//////////////////////////
@@ -398,7 +500,9 @@ namespace Beagle.Daemon {
 
 		public TextReader GetTextReader ()
 		{
-			return new PullingReader (new PullingReader.Pull (PullText));
+			PullingReader pr = new PullingReader (new PullingReader.Pull (PullText));
+			pr.Identifier = Identifier;
+			return pr;
 		}
 
 		public TextReader GetHotTextReader ()
@@ -409,110 +513,5 @@ namespace Beagle.Daemon {
 		public IEnumerable Properties {
 			get { return propertyPool; }
 		}
-
-		//////////////////////////
-
-		static SortedList registry = null;
-
-		static private int ScanAssemblyForFilters (Assembly assembly)
-		{
-			int count = 0;
-
-			foreach (Type t in assembly.GetTypes ()) {
-				if (t.IsSubclassOf (typeof (Filter)) && ! t.IsAbstract) {
-					Filter filter = (Filter) Activator.CreateInstance (t);
-					bool first = true;
-					foreach (Flavor flavor in filter.SupportedFlavors) {
-						if (registry.ContainsKey (flavor)) {
-							Type otherType = (Type) registry [flavor];
-							Filter other = (Filter) Activator.CreateInstance (otherType);
-							String estr = String.Format ("Type Collision: {0} ({1} vs {2})",
-										     flavor, filter, other);
-							throw new Exception (estr);
-						}
-						registry [flavor] = t;
-						if (first) {
-							++count;
-							first = false;
-						}
-					}
-				}
-			}
-
-			return count;
-		}
-
-		static private void FindAssemblies (string dir)
-		{
-			if (dir == null || dir == "")
-				return;
-
-			if (! Directory.Exists (dir)) {
-				//Logger.Log.Debug ("'{0}' is not a directory: No filters loaded", dir);
-				return;
-			}
-
-			DirectoryInfo dirInfo = new DirectoryInfo (dir);
-			foreach (FileInfo file in dirInfo.GetFiles ()) {
-				if (file.Extension == ".dll") {
-					Assembly a = Assembly.LoadFrom (file.FullName);
-					int n = ScanAssemblyForFilters (a);
-					//Logger.Log.Debug ("Loaded {0} filters from {1}", n, file.FullName);
-				}
-			}
-		}
-
-		static private void AutoRegisterFilters ()
-		{
-			string path = Environment.GetEnvironmentVariable ("BEAGLE_FILTER_PATH");
-			
-			if (path == null || path == "")
-				path = PathFinder.FilterDir;
-			else if (path [path.Length-1] == ':')
-				path += PathFinder.FilterDir;
-
-			foreach (string dir in path.Split (':'))
-				FindAssemblies (dir);
-		}
-
-		static public bool CanFilter (Flavor flavor)
-		{
-			return FromFlavor (flavor) != null;
-		}
-
-		static public Filter FromFlavor (Flavor flavor)
-		{
-			if (registry == null) {
-				registry = new SortedList ();
-				AutoRegisterFilters ();
-			}
-
-			if (flavor.IsPattern)
-				throw new Exception ("Can't create filter from content type pattern " + flavor);
-
-			Filter filter = null;
-
-			foreach (Flavor other in registry.Keys) {
-				if (other.IsMatch (flavor)) {
-					Type t = (Type) registry [other];
-					filter = (Filter) Activator.CreateInstance (t);
-					filter.flavor = flavor;
-				}
-			}
-			
-			return filter;
-		}
-
-		static public Filter FilterFromMimeType (String mimeType)
-		{
-			Flavor flavor = Flavor.FromMimeType (mimeType);
-			return FromFlavor (flavor);
-		}
-
-		static public Filter FilterFromPath (String path)
-		{
-			Flavor flavor = Flavor.FromPath (path);
-			return FromFlavor (flavor);
-		}
 	}
 }
