@@ -24,11 +24,15 @@
 // DEALINGS IN THE SOFTWARE.
 //
 
+#define DBUS_IS_BROKEN_BROKEN_BROKEN
+
 using System;
 using System.Collections;
 using System.IO;
 using System.Text;
 
+using Beagle.Util;
+
 namespace Beagle.Daemon {
 
 	public class QueryImpl : Beagle.QueryProxy, IDisposable, IDBusObject {
@@ -37,7 +41,7 @@ namespace Beagle.Daemon {
 		private QueryResult result = null;
 		private string id;
 
-		private Hashtable allHits = new Hashtable ();
+		private Hashtable allHits = UriFu.NewHashtable ();
 		
 		public override event StartedHandler StartedEvent;
 		public override event HitsAddedAsBinaryHandler HitsAddedAsBinaryEvent;
@@ -52,7 +56,6 @@ namespace Beagle.Daemon {
 		{
 			this.id = id;
 
-
 			body = new QueryBody ();
 		}
 
@@ -138,7 +141,11 @@ namespace Beagle.Daemon {
 		public override void Start ()
 		{
 			if (StartedEvent != null)
+#if DBUS_IS_BROKEN_BROKEN_BROKEN
+				HoistStartedEvent ();
+#else
 				StartedEvent (this);
+#endif
 
 			AttachResult ();
 
@@ -159,6 +166,35 @@ namespace Beagle.Daemon {
 				ClosedEvent (this);
 		}
 
+		public override string GetSnippetFromUriString (string uri_string)
+		{
+			string snippet;
+			
+			Uri uri = new Uri (uri_string, false);
+			Hit hit = result.GetHitFromUri (uri);
+
+			if (hit == null) {
+				snippet = "ERROR: invalid hit, uri=" + uri;
+
+				Logger.Log.Debug ("*** Got invalid hit: uri={0}", uri);
+				Logger.Log.Debug ("*** Valid Hits:");
+				foreach (Uri x in result.HitUris)
+					Logger.Log.Debug ("***    {0}", x);
+
+			} else {
+				Queryable queryable = hit.SourceObject as Queryable;
+				if (queryable == null)
+					snippet = "ERROR: hit.SourceObject is null, uri=" + uri;
+				else
+					snippet = queryable.GetSnippet (body, hit);
+			}
+
+			if (snippet == null)
+				snippet = "";
+
+			return snippet;
+		}
+
 		public void Dispose ()
 		{
 			DisconnectResult ();
@@ -190,6 +226,131 @@ namespace Beagle.Daemon {
 
 		//////////////////////////////////////////////////////
 
+#if DBUS_IS_BROKEN_BROKEN_BROKEN
+		//
+		// This works around d-bus bugs by hoisting signal emissions
+		// into the main loop.
+		//
+
+		public void FireStartedEvent ()
+		{
+			StartedEvent (this);
+		}
+
+		public void FireCancelledEvent ()
+		{
+			CancelledEvent (this);
+		}
+
+		public void FireFinishedEvent ()
+		{
+			FinishedEvent (this);
+		}
+
+		public void FireHitsAddedAsBinaryEvent (string arg)
+		{
+			HitsAddedAsBinaryEvent (this, arg);
+		}
+
+		public void FireHitsSubtractedAsStringEvent (string arg)
+		{
+			HitsSubtractedAsStringEvent (this, arg);
+		}
+
+		private class SignalHoister {
+
+			public enum SignalType {
+				Started,
+				Cancelled,
+				Finished,
+				HitsAdded,
+				HitsSubtracted
+			}
+
+			public SignalType Type;
+			public QueryImpl Sender;
+			public string Arg;
+
+			private bool IdleHandler ()
+			{
+				switch (this.Type) {
+
+				case SignalType.Started:
+					Sender.FireStartedEvent ();
+					break;
+
+				case SignalType.Cancelled:
+					Sender.FireCancelledEvent ();
+					break;
+
+				case SignalType.Finished:
+					Sender.FireFinishedEvent ();
+					break;
+
+				case SignalType.HitsAdded:
+					Sender.FireHitsAddedAsBinaryEvent (Arg);
+					break;
+
+				case SignalType.HitsSubtracted:
+					Sender.FireHitsSubtractedAsStringEvent (Arg);
+					break;
+				}
+
+				return false;
+			}
+
+			public void Run ()
+			{
+				GLib.Idle.Add (new GLib.IdleHandler (IdleHandler));
+			}
+		}
+
+		public void HoistStartedEvent ()
+		{
+			SignalHoister signal = new SignalHoister ();
+			signal.Type = SignalHoister.SignalType.Started;
+			signal.Sender = this;
+			signal.Run ();
+		}
+
+		public void HoistCancelledEvent ()
+		{
+			SignalHoister signal = new SignalHoister ();
+			signal.Type = SignalHoister.SignalType.Cancelled;
+			signal.Sender = this;
+			signal.Run ();
+		}
+
+		public void HoistFinishedEvent ()
+		{
+			SignalHoister signal = new SignalHoister ();
+			signal.Type = SignalHoister.SignalType.Finished;
+			signal.Sender = this;
+			signal.Run ();
+		}
+
+		public void HoistHitsAddedAsBinaryEvent (string arg)
+		{
+			SignalHoister signal = new SignalHoister ();
+			signal.Type = SignalHoister.SignalType.HitsAdded;
+			signal.Sender = this;
+			signal.Arg = arg;
+			signal.Run ();
+		}
+
+		public void HoistHitsSubtractedAsStringEvent (string arg)
+		{
+			SignalHoister signal = new SignalHoister ();
+			signal.Type = SignalHoister.SignalType.HitsSubtracted;
+			signal.Sender = this;
+			signal.Arg = arg;
+			signal.Run ();
+		}
+#endif
+				
+
+		//////////////////////////////////////////////////////
+
 		//
 		// QueryResult event handlers
 		//
@@ -227,11 +388,23 @@ namespace Beagle.Daemon {
 				allHits[hit.Uri] = hit;
 			}
 			
-			if (HitsSubtractedAsStringEvent != null && toSubtract.Count > 0)
-				HitsSubtractedAsStringEvent (this, UrisToString (toSubtract));
-
-			if (HitsAddedAsBinaryEvent != null && someHits.Count > 0)
-				HitsAddedAsBinaryEvent (this, HitsToBinary (someHits));
+			if (HitsSubtractedAsStringEvent != null && toSubtract.Count > 0) {
+				string uri_str = UriFu.UrisToString (toSubtract);
+#if DBUS_IS_BROKEN_BROKEN_BROKEN
+				HoistHitsSubtractedAsStringEvent (uri_str);
+#else
+				HitsSubtractedAsStringEvent (this, UriFu.UrisToString (toSubtract));
+#endif
+			}
+			
+			if (HitsAddedAsBinaryEvent != null && someHits.Count > 0) {
+				string hits_binary = HitsToBinary (someHits);
+#if DBUS_IS_BROKEN_BROKEN_BROKEN
+				HoistHitsAddedAsBinaryEvent (hits_binary);
+#else
+				HitsAddedAsBinaryEvent (this, hits_binary);
+#endif
+			}
 
 		}
 
@@ -241,20 +414,11 @@ namespace Beagle.Daemon {
 				return;
 
 			if (FinishedEvent != null) 
+#if DBUS_IS_BROKEN_BROKEN_BROKEN
+				HoistFinishedEvent ();
+#else
 				FinishedEvent (this);
-		}
-
-		private string UrisToString (ICollection uris)
-		{
-			StringBuilder builder = null;
-			foreach (Uri uri in uris) {
-				if (builder == null)
-					builder = new StringBuilder ("");
-				else
-					builder.Append ("|");
-				builder.Append (uri.ToString ());
-			}
-			return builder  != null ? builder.ToString () : "";
+#endif
 		}
 
 		private void OnHitsSubtractedFromResult (QueryResult source, ICollection someUris)
@@ -270,8 +434,14 @@ namespace Beagle.Daemon {
 					allHits.Remove (uri);
 				}
 			}
-			if (HitsSubtractedAsStringEvent != null && toSubtract.Count > 0)
-				HitsSubtractedAsStringEvent (this, UrisToString (toSubtract));			
+			if (HitsSubtractedAsStringEvent != null && toSubtract.Count > 0) {
+				string uri_str = UriFu.UrisToString (toSubtract);
+#if DBUS_IS_BROKEN_BROKEN_BROKEN
+				HoistHitsSubtractedAsStringEvent (uri_str);
+#else
+				HitsSubtractedAsStringEvent (this, uri_str);
+#endif
+			}
 		}
 
 		private void OnCancelledResult (QueryResult source) 
@@ -280,7 +450,11 @@ namespace Beagle.Daemon {
 				return;
 
 			if (CancelledEvent != null)
+#if DBUS_IS_BROKEN_BROKEN_BROKEN
+				HoistCancelledEvent ();
+#else
 				CancelledEvent (this);
+#endif
 		}
 
 		//////////////////////////////////////////////////////
@@ -291,8 +465,8 @@ namespace Beagle.Daemon {
 
 		private void OnQueryDriverChanged (Queryable queryable, IQueryableChangeData changeData)
 		{
-			if (result != null && queryable.AcceptQuery (body))
-				queryable.DoQuery (body, result, changeData);
+			if (result != null)
+				QueryDriver.DoOneQuery (queryable, body, result, changeData);
 		}
 	}
 }
